#!/bin/bash

#vytvoří a uloží pravidla firewallu pro pozdější znovunačtení

#--------------------#
#     PARAMETERY     #
#--------------------#

OUTPUT_FILE_4=/etc/network/ip4tables.txt
OUTPUT_FILE_6=/etc/network/ip6tables.txt

LAN=enp1s0
WAN=ppp0
IPv4_LOCAL_NET=10.0.0.0/24 #naše lokální IPv4 podsíť
IPv4_PUBLIC_ADDRESS=109.81.195.224 #pevná, veřejná IPv4 adresa (není-li tato pevná, je potřeba se zaobírat MASQUERADE)
IPv4_INTERNALSERVER_ADDRESS=10.0.0.1 #IPv4 adresa nějakého vnitřního serveru pro úplnost příkladu
IPv4_TABLES_MANIPULATOR=/usr/sbin/iptables
IPv4_TABLES_SAVE=/usr/sbin/iptables-save
IPv6_TABLES_MANIPULATOR=/usr/sbin/ip6tables
IPv6_TABLES_SAVE=/usr/sbin/ip6tables-save
#porty služeb, které chceme mít dostupné - pro úplnost příkladu, nechť běží mail-server na jiném stroji ve vnitřní síti
SRV_HTTP=80 #web
SRV_HTTPS=443 #secured web
SRV_SMTP=25 #e-mail

#soubory pro uložení pravidel lze zadat na příkazovou řádku
if [ ! -z "$1" ]; then
	OUTPUT_FILE_4=$1
fi

if [ ! -z "$2" ]; then
	OUTPUT_FILE_6=$2
fi

echo 'output files'
echo "   IPv4 - $OUTPUT_FILE_4"
echo "   IPv6 - $OUTPUT_FILE_6"

#-------------------#
#       FUNKCE      #
#-------------------#

#provede příkaz a pokud tento selže, skončí s návratovým kódem 255
issue_command () {
	eval $@
	if [ $? -ne 0 ]; then
		echo "command failed: '$@'"
		echo 'FATAL, exiting'
		exit 255
	fi
}

#vytvoří IPv4 pravidlo
create_v4_rule () {
	issue_command ${IPv4_TABLES_MANIPULATOR} $@
}

#vytvoří IPv6 pravidlo
create_v6_rule () {
	issue_command ${IPv6_TABLES_MANIPULATOR} $@
}

#vytvoří dual-stack pravidlo
create_dual_rule () {
	create_v4_rule $@
	create_v6_rule $@
}

#--------------------#
#      PRAVIDLA      #
#--------------------#

#vyprázdníme tabulky, aby nezůstaly nějaké resty
create_dual_rule -F
create_dual_rule -t nat -F

#IPv4 forwarding se podobně jako IPv6 musí zapnout v /etc/sysctl.conf, leč kvůli nutnosti přepisování zdrojové, nevěřejné adresy na veřejnou
#potřebujeme kromě sysctl ještě routovací pravidlo (nepotřebujeme MASQUERADE, protože v našem případě je IPv4 adresa pevná)
create_v4_rule -t nat -A POSTROUTING ! -d ${IPv4_LOCAL_NET} -o ${WAN} -j SNAT --to-source ${IPv4_PUBLIC_ADDRESS}

#pro úplnost: základní řetězce jsou INPUT - pakety pro tento počítač, OUTPUT - pakety odesílané tímto počítačem, a FORWARD - pakety pro jiné počítače
#nastavíme výchozí politiku řetězců INPUT, OUTPUT a FORWARD - pokud paket nesplní žádné z pravidel, skončí
create_dual_rule -P INPUT DROP #zahozen
create_dual_rule -P FORWARD DROP
create_dual_rule -P OUTPUT ACCEPT #či přijat

#povolíme zpětnou smyčku, vše zevnitř a ze vně jen to, co bylo vyžádáno zevnitř
create_dual_rule -A INPUT -i lo -j ACCEPT
create_dual_rule -A INPUT -i $LAN -j ACCEPT
create_dual_rule -A FORWARD -i $LAN -j ACCEPT
create_dual_rule -A INPUT -m conntrack --ctstate ESTABLISHED,RELATED -j ACCEPT
create_dual_rule -A FORWARD -m conntrack --ctstate ESTABLISHED,RELATED -j ACCEPT

#invalidní pakety zahodíme
create_dual_rule -A INPUT -m conntrack --ctstate INVALID -j DROP
create_dual_rule -A FORWARD -m conntrack --ctstate INVALID -j DROP

#nyní povolíme služby - web na tomto počítači a mail na počítači uvnitř LAN, abychom pro úplnost ukázali i příklad tzv. port-forwardingu
#port se forwarduje ve formátu <CÍL>:<PORT>, takže není problémem, běží-li služba uvnitř na nestandardním portu
create_v4_rule -i $WAN -t nat -A PREROUTING -p tcp --dport ${SRV_SMTP} -j DNAT --to-destination ${IPv4_INTERNALSERVER_ADDRESS}:${SRV_SMTP}
#přepsání cílové adresy přehodí paketu vyhybku a místo v INPUT skončí v řetězci FORWARD, ale pokud bychom port nepřipustili, byl by tu zahozen
#proto pakety s daným cílovým portem necháme připustit - IPv4 i IPv6 (IPv6 nepotřebuje prerouting, protože adresy jsou veřejné)
create_dual_rule -A FORWARD -p tcp --dport ${SRV_SMTP} -j ACCEPT
#web běží na tomto počítači
create_dual_rule -A INPUT -p tcp -m multiport --dports ${SRV_HTTP},${SRV_HTTPS} -j ACCEPT

#DHCP6 používá UDP porty 546 na klientské a 547 na serverové straně - aby přidělení prefixu fungovalo, musíme tyto povolit
create_v6_rule -A INPUT -m state --state NEW -m udp -p udp --dport 546 --sport 547 -s fe80::/10 -d fe80::/10 -j ACCEPT

#IPv6 - na rozdíl od IPv4 - potřebuje určité ICMP pakety ke svému provozu
#propagace chyb
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 1 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 2 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 3 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 4 -j ACCEPT
create_v6_rule -A FORWARD -p icmpv6 --icmpv6-type 1 -j ACCEPT
create_v6_rule -A FORWARD -p icmpv6 --icmpv6-type 2 -j ACCEPT
create_v6_rule -A FORWARD -p icmpv6 --icmpv6-type 3 -j ACCEPT
create_v6_rule -A FORWARD -p icmpv6 --icmpv6-type 4 -j ACCEPT
#hledání routerů a sousedů
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 133 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 134 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 135 -j ACCEPT
create_v6_rule -A INPUT -p icmpv6 --icmpv6-type 136 -j ACCEPT

#hotovo, teď pravidla exportujeme do souborů
issue_command "${IPv4_TABLES_SAVE} > ${OUTPUT_FILE_4}"
issue_command "${IPv6_TABLES_SAVE} > ${OUTPUT_FILE_6}"
